//----------------------------------------------------------------
//  Time-stamp: <99/08/04 13:05 sutner> 
//				<00/09/17 01:24 mjs>
//
//  File:      graph-driver.cc
//  Purpose:   driver for 113 Lab 2      
//----------------------------------------------------------------

#include  <iostream>
#include  <fstream>
#include  "graph-list.h"
using namespace std;

// function declarations
int MaxOutDegree(const Graph &g);
int MinOutDegree(const Graph &g);
int MaxInDegree(const Graph &g);
int MinInDegree(const Graph &g);
int MaxDegree(const Graph &g);
int MinDegree(const Graph &g);


int  main(int argc, char * argv[])
{
	if (argc != 2)
	{
		cerr << "Usage: " << argv[0] << " <graph-file> " << endl;
		exit(0);
	}
	
	ifstream infile(argv[1]);	// bind input file to stream
	
	if (!infile)
	{
		cerr << "cannot open " << argv[1] <<  " for input" << endl;
		exit(1);
	}
	
	//Load a Graph
	Graph G;
	int numV, a, b, cost;
	infile >> numV;
	
	G.SetSize(numV);
	while(infile >> a >> b >> cost)
		G.AddEdge(a, b, cost);
	
	cout << "Graph has " << G.NumNodes() << " vertices ";
	cout << "and " << G.NumEdges() << " edges:" << endl << endl;
	cout << "Initial graph:\n" << G << endl;
	G.RemoveEdge(0,6);
	cout << G << endl;
	G.RemoveEdge(4,5);
	cout << G << endl;
	G.RemoveEdge(6,8);
	cout << G << endl;
	G.RemoveEdge(6,7);
	cout << G << endl;
	G.RemoveEdge(6,9);
	cout << G << endl;
	G.AddEdge(4,6,50);
	cout << G << endl << endl;
	
	cout << "Graph now has " << G.NumNodes() << " vertices ";
	cout << "and " << G.NumEdges() << " edges:" << endl << endl;
	
	if (G.HasEdge(0,1))
		cout << "(0,1) present with cost " << G.EdgeValue(0,1) << endl;
	else
		cout << "(0,1) not present" << endl;
	
	if (G.HasEdge(8,5))
		cout << "(8,5) present with cost " << G.EdgeValue(8,5) << endl;
	else
		cout << "(8,5) not present" << endl;
	cout << endl << endl;
	
	cout << "Stats: \n";
	cout << "  Max Out Degree = " << MaxOutDegree(G) << endl;
	cout << "  Min Out Degree = " << MinOutDegree(G) << endl;
	cout << "  Max In  Degree = " << MaxInDegree(G) << endl;
	cout << "  Min In  Degree = " << MinInDegree(G) << endl;
	cout << "  Max Degree     = " << MaxDegree(G) << endl;
	cout << "  Min Degree     = " << MinDegree(G) << endl;
	
	return 0;
}

//---------------------------------------------------------------------------
//Extremal Degree Functions
//---------------------------------------------------------------------------

int MaxOutDegree (const Graph &g)
{
	int answer = 0;
	for (int i = 0; i < g.NumNodes(); ++i)
	{
		int temp = g.OutDegree(i);
		if (temp > answer)
			answer = temp;
	}
  
	return answer;
}

int MinOutDegree (const Graph &g)
{
	int answer = g.NumNodes();
	for (int i = 0; i < g.NumNodes(); ++i)
	{
		int temp = g.OutDegree(i);
		if (answer > temp)
			answer = temp;
	}
  
	return answer;
}

int MaxInDegree (const Graph &g)
{
	int answer = 0;
	for (int i = 0; i < g.NumNodes(); ++i)
	{
		int temp = g.InDegree(i);
		if (answer < temp)
			answer = temp;
	}
  
	return answer;
}

int MinInDegree (const Graph &g)
{
	int answer = g.NumNodes();
	for (int i = 0; i < g.NumNodes(); ++i)
	{
		int temp = g.InDegree(i);
		if (answer > temp)
			answer = temp;
	}
  
	return answer;
}

int MaxDegree (const Graph &g)
{
	int answer = 0;
	for (int i = 0; i < g.NumNodes(); ++i)
	{
		int temp = g.Degree(i);
    	if (answer < temp)
			answer = temp;
	}
  
	return answer;
}

int MinDegree (const Graph &g)
{
	int answer = g.NumNodes();
	for (int i = 0; i < g.NumNodes(); ++i)
	{
    	int temp = g.Degree(i);
    	if (answer > temp)
			answer = temp;
	}
  
  return answer;
}
